#ifndef _KFIR_H_
#define _KFIR_H_

/* For 2.2.x backwards compatibility */
#if LINUX_VERSION_CODE < 0x020300
#define net_device device
#define DECLARE_MUTEX(foo)         struct semaphore foo = MUTEX
#define DECLARE_MUTEX_LOCKED(foo)  struct semaphore foo = MUTEX_LOCKED
#define WAIT_QUEUE                 struct wait_queue*
#define init_waitqueue_head(wq)    *(wq) = NULL;
#define DECLARE_WAITQUEUE(wait, current) struct wait_queue wait = { current, NULL }
#define set_current_state(state_value)			\
	do { current->state = state_value; } while (0)
#else
#define WAIT_QUEUE                 wait_queue_head_t
#endif

typedef u8 BYTE;
typedef u16 WORD;
typedef u32 DWORD;
typedef u64 QWORD;
typedef int bool;
typedef bool BOOL;

#include "KfirDef.h"
#include "KfirCMD.h"

#define MAX_NUM_KFIR 8

#define wplxl(adr, dat)    writel((dat), (adr)+kfir->mem0)
#define rplxl(adr)         readl((adr)+kfir->mem0)

#define waltl(adr, dat)    writel((dat), kfir->AlteraReg+kfir->adr)
#define raltl(adr)         readl(kfir->AlteraReg+kfir->adr)

#define KfirRegWrite(adr, dat) (writel((dat), kfir->KfirReg+adr), udelay(10))
#define KfirRegRead(adr)       (udelay(10),readl(kfir->KfirReg+adr))

#define AlteraRegWrite(adr, dat) (writel((dat), kfir->AlteraReg+adr), udelay(10))
#define AlteraRegRead(adr)       (udelay(10),readl(kfir->AlteraReg+adr))

// kfir status.
typedef struct KFIR_INT_REG_Tag {
        WORD        IntCnt;             // 0  - 221
        WORD        OldIntCnt;
        //WORD        AudioPTS_0;         // 1  - 222
        //WORD        AudioPTS_1;         // 2  - 223
        //WORD        AudioPTS_2;         // 3  - 224
        WORD        GOP_0;              // 4  - 128
        WORD        GOP_1;              // 5  - 129
        WORD        Frame;              // 6  - 183
        WORD        PictType;           // 7  - 132
        WORD        MuxCnt_0;           // 8  - 231
        WORD        MuxCnt_1;           // 9  - 232
        WORD        MuxCnt_2;           // 10 - 233
        //WORD        IntAck;             // 11 - 228
        WORD        FIFO_Reg_0;         // 12 - 184
        WORD        FIFO_Reg_1;         // 13 - 185
        WORD        SRAM_Reg_0;         // 14 - 188
        WORD        SRAM_Reg_1;         // 15 - 189
        WORD        SRAM_Cnt_0;         // 16 - 186
        WORD        SRAM_Cnt_1;         // 17 - 187
        WORD        KfirEReady;         // 18 - 254
        //WORD        V11Record;          // 19 - 255
        //WORD        Clk27M_LS;          // 20 - 
        //WORD        Clk27M_MS;          // 21 - 
        WORD        IntVideo;           // 22 - 234
        WORD        IntAudio;           // 23 - 235
        WORD        IntDMA_Mux;         // 24 - 236
        WORD        IntDMA_VIB;         // 25 - 237
} KFIR_INT_REG;


struct kfir_dev {
        struct video_device video;
        struct i2c_client *decoder;

        struct pci_dev *pdev;
        int mode;
        unsigned char revision;
        unsigned short subsystem_device;
        

        u8* dmabuf;
        u8* mpgbuf; 
#define MPGBUFSIZE 256*1024
  
        int readp;
        int writep;
        int recording;

	WAIT_QUEUE wq;
  
        unsigned char *mem0;
        unsigned char *mem1;
        unsigned char *mem2;
        unsigned char *mem3;

        unsigned char *KfirReg;
        unsigned char *AlteraReg;
          
        u32 ALT_ADR_INT_MASK;
        u32 ALT_ADR_INT_REG_CLR;
        u32 ALT_ADR_KFIR_AUDIO_BASE;
        u32 ALT_ADR_RESET_REG;

	u32 ALT_ADR_ADSP_ADDR;
	u32 ALT_ADR_ADSP_DATA;
	u32 ALT_ADR_ADSP_IRQ;
	u32 ALT_ADR_ADSP_FREQ;
	u32 ALT_ADR_I2C;
	u32 ALT_ADR_CONTROL_0;
	u32 ALT_ADR_CONTROL_1;
	u32 ALT_ADR_CLOCK_PADI;

	/* i2c */
	struct i2c_adapter         i2c_adap;
	struct i2c_algo_bit_data   i2c_algo;
	struct i2c_client          i2c_client;
        u32 i2c;
        int i2c_ok;

        int kfirreset;

        u64 AudioStreamPos;
        WORD m_CmdReg;
       

        int WasReset;
        int FlFirstTime;
        int FlValid;
        int FlStopVideo;
        int MajorVersion;
        int MinorVersion;
        int DVB_MODE;

        u32 *micro;
        int mp;
  
        WORD Par[KFIR_MAX_PARAMS];
        WORD MuxParams[MAX_MUX_PARAMS];
        u8 Table[ TRANSPORT_PACKET_SIZE ];
        VT_KFIR_CParams Params;
        KFIR_INT_REG KfState;
        QWORD TotalLinesAfterMUX;
        u32 IntVideoCnt;
        u32 IntAudioCnt;
        u32 IntVibCnt;
        int LastDMASize;
        u32 EventMask;
        u32 TotalLinessaved;
};

// Defines for the Revision_ID.
#define KFIR_BOARD_A      0xA
#define KFIR_BOARD_B      0xB
#define KFIR_BOARD_C      0xC
#define KFIR_BOARD_D      0xD
#define KFIR_BOARD_E      0xE
#define KFIR_BOARD_5E     0x5E

// Defines for the Subsystem_ID;
#define KFIR_MASTER_MODE_ID  0x0100
#define KFIR_GENERIC_MODE_ID 0x0200

// Kfir registers 
#define ALT_ADR_KFIR_REV_5E         0x2000

// 1 - Source of interrupt on Read
// 1 - Clear Interrupt when Write
#define ALT_ADR_INT_REG_CLR_REV_5E  0x3000
#define ALT_D_INT_REQ_CLR_KFIR_INT          0x0001
#define ALT_D_INT_REQ_CLR_ADSP_INT          0x0002
#define ALT_D_INT_CLEAR_ALL                 0xFF

 
#define ALT_ADR_I2C_SLAVE_SHIFT             16


#define ALT_D_I2C_READY                     0x1000



// 1 - Mask this interupt when Write
#define ALT_ADR_INT_MASK_REV_5E     0x3004
// see ALT_ADR_INT_REG_CLR


#define ALT_ADR_CLOCK_PADI_DEF          0x302C


// 1 - Reset the device when Write 0, except for Kfir PLL - 1
#define ALT_ADR_RESET_REG_REV_5E    0x3008
#define ALT_D_RESET_REG_KFIR_REV_5E          0x0001
#define ALT_D_RESET_REG_KFIR_PLL_REV_5E      0x0002


// ADSP address when write - IAL
#define ALT_ADR_ADSP_ADDR_REV_5E 0x300C

// ADSP data when read/write - IDMA
#define ALT_ADR_ADSP_DATA_REV_5E    0x3010


// Set ADSP IRQs (write 0, then 1) - GPPORT
#define ALT_ADR_ADSP_IRQ_REV_5E     0x3014
#define ALT_D_ADSP_IRQ_E                    0x0001
#define ALT_D_ADSP_IRQ_2                    0x0002


#define ALT_ADR_AUDIO_FREQ_REV_5E   0x3018


// I2C interface.
#define ALT_ADR_I2C_REV_5E          0x301C
#define ALT_ADR_I2C_CLK_REV_5E               0x0001
#define ALT_ADR_I2C_DATA_REV_5E              0x0002

#define ALT_ADR_I2C_DATA_MASK               0x000F


#define ALT_ADR_OUTPUT_REV_5E       0x3020
#define ALT_D_SERIAL                          0x0
#define ALT_D_PARALLEL                        0x1
#if 1 /* FIXME */
#define ALT_D_DMA                             0x2
#else
#define ALT_D_DMA                             0x1
#undef ALT_D_PARALLEL
#define ALT_D_PARALLEL                        0x2
#endif
#define ALT_D_OUTPUT_MASK                     0x3


#define AUDIO_FRAME_TIME_32_000 36
#define AUDIO_FRAME_TIME_48_000 24

#define KFIR_FILE_LENGTH 600
#define KFIR_MIC_DATE_STAMP_LENGTH  32


#ifndef PCI_VENDOR_ID_VISIONTECH
#define PCI_VENDOR_ID_VISIONTECH		0x141f
#endif
#ifndef PCI_DEVICE_ID_VISIONTECH_KFIR_5E
#define PCI_DEVICE_ID_VISIONTECH_KFIR_5E	0x8000
#endif

// Type of the extension.
typedef enum KFIR_EXT_TYPE_TAG {
  KFIR_EXT_BOARDS = 0,
  KFIR_EXT_ENCODER,
} KFIR_EXT_TYPE;

#define ADR0              0x7fd8
#define ADR1              (ADR0+0x0001)   // 0x0x7fd9
#define ADR2              (ADR0+0x0002)   // 0x0x7fdA
#define ADR3              (ADR0+0x0003)   // 0x0x7fdB
#define ADR4              (ADR0+0x0004)   // 0x0x7fdC
#define ADR5              (ADR0+0x0005)   // 0x0x7fdD


// Memory for the DMA or Kfir Master Mode Buffer size.
#define ONE_MEGA                  0x100000
#define KF_DMA_BUFFER_SIZE        ((ONE_MEGA) / 2)
#define KF_VIB_DMA_BUFFER_SIZE    ((ONE_MEGA) / 2)
// Size of the FIFO audio buffer (from ADSP to Kfir) for the Program/Transport stream.
#define KF_FIFO_BUFFER_SIZE       0x8000
// TODO: Should be deleted to optimize speed.
#define KF_TEMP_AUDIO_BUFFER_SIZE ((KF_AUDIO_MEM_SIZE) * 4)
// Size for the ADSP encoded stream
#define KF_AUDIO_BUFFER_SIZE      (0x10000/2)

// Number of the buffers to allocate.
#define KF_NUM                8
#if KF_DMA_BUFFER_SIZE != (ONE_MEGA / 2)
#error  Please think a little and redefine the KF_NUM value
#endif

// Define for the Length of the MUX DMA burst.
#define MUX_DMA_LENGTH        (0x4000)
// 0x1000 - not stable on Dell Precision 610
// 0x2000 - stable
// 0x4000 - stable
// 0x20000 - good for 5E board.
#if (DMA_BUFFER_SIZE % MUX_DMA_LENGTH) != 0
#error "KFIR driver realization error: DMA_BUFFER_SIZE must be devisible by LENGTH"
#endif

#define MAX_FIFO_CHUNK  (32 * 1024)

// Defines for the FILE mode - TODO: Delete them ?
#define VT_FILE_BUFFERS_NUM		0xC
#define VT_VIDEO_BUFFER_SIZE  (0x10000 / 2)


// from Kfir.h

typedef enum KF_CONTR_TYPE_Tag {
        KF_GC               = 0,
        KF_DSP_RISC_MC      = 1,
        KF_DSP_RISC_CRAM    = 2,
        KF_DSP_UNIT_MC      = 3,
        KF_BSM_MC           = 4,
        KF_MUX_MC           = 5,
} KF_CONTR_TYPE;
const DWORD MAX_BUFFER = 16384;

typedef enum KF_CONTR_TYPE_DEV_ID_Tag {
        KF_DEV_ID_GC             = 7,
        KF_DEV_ID_DSP            = 4,
        KF_DEV_ID_BSM            = 5,
        KF_DEV_ID_MUX            = 8,
} KF_CONTR_DEV_ID;

const int KF_MIC_DATE_STAMP_LEN = 40;
const int KF_MIC_TIME_STAMP_LEN = 40;
const char KF_MIC_DELIMETER = '#';

#define ISO13818_VIDEO	2

#define IDR_MICROCODE_D                 129
#define IDR_MICROCODE_D_LEN             130
#define IDR_MICROCODE_E                 131
#define IDR_MICROCODE_E_LEN             132
#define IDR_MICROCODE_E_FM              133
#define IDR_MICROCODE_E_FM_LEN          134
#define IDR_KFIR_GC_MC                  139
#define IDR_KFIR_DSP_RISC_MC            140
#define IDR_KFIR_DSP_RISC_CRAM          141
#define IDR_KFIR_DSP_UNIT_MC            142
#define IDR_KFIR_BSM_MC                 143
#define IDR_KFIR_MUX_MC                 144
#define IDR_ALTERA_RBF1                 147
#define IDR_ALTERA_RDK_8000AB           147
#define IDR_ALTERA_RDK_8000AB_LEN       148
#define IDR_ALTERA_RDK_8000C            149
#define IDR_ALTERA_RDK_8000C_LEN        150
#define IDR_ALTERA_RDK_8000E            152
#define IDR_ALTERA_RDK_8000E_LEN        153
#define IDR_ALTERA_RDK_80005E           154
#define IDR_ALTERA_RDK_80005E_LEN       155


/* ADSP defines */ 

#define IAL               0
#define IDMA              1
#define GPPORT            2
#define ADR0              0x7fd8
#define ADR1              (ADR0+0x0001)
#define ADR2              (ADR0+0x0002)
#define ADR3              (ADR0+0x0003)
#define ADR4              (ADR0+0x0004)
#define ADR5              (ADR0+0x0005)
#define BOARDPRESENTMASK  0x4

enum AudioCommands
{
        DSPAPI_VERIFYSYSTEM             = 0x0001,
	DSPAPI_QUERYSUPPORTPLAY         = 0x0002,
	DSPAPI_SETSTYLE                 = 0x0003,
	DSPAPI_SETSAMPLINGRATE          = 0x0004,
	DSPAPI_SETCHANNELS              = 0x0005,
	DSPAPI_SETRESOLUTION            = 0x0006,
	DSPAPI_SETCRCOPTIONS            = 0x0007,
	DSPAPI_BUFENQFOR                = 0x0008,
	DSPAPI_LOGBUFFER                = 0x0009,
	DSPAPI_STARTPLAY                = 0x000a,
	DSPAPI_STOPPLAY                 = 0x000b,
	DSPAPI_AUTOSTOP                 = 0x000c,
	DSPAPI_STARTRECORD              = 0x000d,
	DSPAPI_STOPRECORD               = 0x000e,
	DSPAPI_GETLASTPROCESSEDBUFFER   = 0x000f,
	DSPAPI_PAUSEPLAYRECORD          = 0x0010,
	DSPAPI_RESTARTPLAYRECORD        = 0x0011,
	DSPAPI_SETVOLUME                = 0x0012,
	DSPAPI_QUERYSUPPORTRECORD       = 0x0013,
	DSPAPI_GENERALBUFENQ            = 0x0014,
	DSPAPI_SETDOWNMIXTYPE           = 0x0015,
	DSPAPI_SETINPUTGAIN             = 0x0016,
	DSPAPI_SETLINEOUT               = 0x0017,
	DSPAPI_SETLANGMIXTYPE           = 0x0018
};

enum AudioSystemCommands
{
        SYSCMD_STARTMASTERDSP           = 0x0001,
	SYSCMD_STOPMASTERDSP            = 0x0004,
	SYSCMD_DISABLEINT               = 0x0020,
	SYSCMD_CANCELMASTERINT          = 0x0040,
	SYSCMD_MASTERDSPINT             = 0x0100,
	SYSCMD_SELECTINT                = 0x0400,
	SYSCMD_DESELECTINT              = 0x0800,
	SYSCMD_RAISEIRQEINT             = 0x1000
};

enum AudioCommandCodes
{
        CLEARIRQLED  = 0xB,
	RAISEIRQ2INT = 0xD,
	RAISEIRQEINT = 0xE,
	RESETALLINTS = 0xF,
	START_DSP    = 0x8,
	CHKBOARD     = 0x4
};
enum AudioStatus
{
        SYSTEM_PRESENT                  = 0x0001,
	SYSTEM_CONFIGURED               = 0x0002,
	SYSTEM_ENABLED                  = 0x0004,
	SYSTEM_ALLOCATED                = 0x0008,
	SYSTEM_PLAYING                  = 0x0010,
	SYSTEM_RECORDING                = 0x0020,
	SYSTEM_PAUSED                   = 0x0040,
	SYSTEM_SUSPENDED                = 0x0080,
	SYSTEM_LOCKED                   = 0x0100
};

#define PLL_MAX_DATA 25

#endif
